#-----------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# WebSphere Commerce
#
# (C) Copyright IBM Corp. 2007, 2013 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#-----------------------------------------------------------------

import re

#-------------------------------------------------------------------------------
# wsadminToList()
#
# Converts the string return value of wsadmin commands to a Jython list. This
# method is generated by the Jacl2Jython tool.
#
# PARAMETERS
#   inStr:              The input string.
#-------------------------------------------------------------------------------
def wsadminToList(inStr):
    outList=[]
    if (len(inStr) > 0 and inStr[0] == '[' and inStr[-1] == ']'):
        inStr = inStr[1:-1]
        tmpList = inStr.split(" ")
    #endIf
    else:
        tmpList = inStr.split("\n")
    #endElse
    for item in tmpList:
        item = item.rstrip()
        if (len(item)>0):
            outList.append(item)
        #endIf
    #endFor
    return outList
#endDef

#-------------------------------------------------------------------------------
# getModule()
#
# Returns the configuration ID of the module with the given URI in the
# application.
#
# PARAMETERS
#   appName:            The application name.
#   moduleURI:          The module URI.
#
# RETURNS
#   The configuration ID of the module if it is found, empty string otherwise.
#-------------------------------------------------------------------------------
def getModule(appName, moduleURI):
    deployment = AdminConfig.getid("/Deployment:" + appName + "/")
    if (deployment != ""):
        deployedObject = AdminConfig.showAttribute(deployment, "deployedObject")
        moduleList = wsadminToList(AdminConfig.showAttribute(deployedObject,
                "modules"))
        for module in moduleList:
            uri = AdminConfig.showAttribute(module, "uri")
            if (moduleURI == uri):
                return module
            #endIf
        #endFor
    #endIf
    return ""
#endDef

#-------------------------------------------------------------------------------
# getTargetServerMapping()
#
# Returns the target server mapping that is used to map modules to servers.
#
# PARAMETERS
#   appName:            The application name.
#   mapToWebServer:     "true" if web server mapping should be included; "false"
#                       otherwise.
#
# RETURNS
#   The target server mapping that is used to map modules to servers.
#-------------------------------------------------------------------------------
def getTargetServerMapping(appName, mapToWebServer):
    targetServerMapping = ""
    cellName = AdminConfig.showAttribute(AdminConfig.getid("/Cell:/"), "name")
    serverTargetList = wsadminToList(AdminConfig.getid("/Deployment:"
            + appName + "/ServerTarget:/"))
    for serverTarget in serverTargetList:
        nodeName = AdminConfig.showAttribute(serverTarget, "nodeName")
        serverName = AdminConfig.showAttribute(serverTarget, "name")
        server = AdminConfig.getid("/Node:" + nodeName + "/Server:"
                + serverName + "/")
	#Check if server is empty
	if not server:
		# server shoudn't be empty. Stop and print an error message
		print("ERROR: The server ID for server \"" + serverName 
		+ "\" for node \"" + nodeName + "\" can't be empty. Check if"
		+ " your envrionment is configured correctly.")
                return 0
	#endIf
        serverType = AdminConfig.showAttribute(server, "serverType")
        if ((serverType == "WEB_SERVER" and mapToWebServer == "true")
                or serverType == "APPLICATION_SERVER"):
            targetServerMapping += ("+WebSphere:cell=" + cellName + ",node="
                    + nodeName + ",server=" + serverName)
        #endIf
    #endFor
    clusteredTargetList = wsadminToList(AdminConfig.getid("/Deployment:"
        + appName + "/ClusteredTarget:/"))
    for clusteredTarget in clusteredTargetList:
        clusterName = AdminConfig.showAttribute(clusteredTarget, "name")
        targetServerMapping += ("+WebSphere:cell=" + cellName + ",cluster="
                + clusterName)
    #endFor
    if (targetServerMapping != ""):
        return targetServerMapping[1:]
    #endIf
#endDef

#-------------------------------------------------------------------------------
# regenPluginCfg()
#
# Regenerates the node plug-in configuration of web servers.
#
# PARAMETERS
#   appName:            The application name.
#   propagate:          "true" if the configuration file should be propagated;
#                       "false" otherwise.
#-------------------------------------------------------------------------------
def regenPluginCfg(appName, propagate):
    if whatEnv() == 'nd':
        generator = AdminControl.completeObjectName("type=PluginCfgGenerator,process=dmgr,*")
    else:
        generator = AdminControl.completeObjectName("type=PluginCfgGenerator,*")
    #endif
    cellName = AdminControl.getCell()
    serverTargetList = wsadminToList(AdminConfig.getid("/Deployment:"
            + appName + "/ServerTarget:/"))
    for serverTarget in serverTargetList:
        nodeName = AdminConfig.showAttribute(serverTarget, "nodeName")
        serverName = AdminConfig.showAttribute(serverTarget, "name")
        server = AdminConfig.getid("/Node:" + nodeName + "/Server:"
                + serverName + "/")
        webContainer = AdminConfig.list("WebContainer", server)
        if (webContainer == ""):
            AdminControl.invoke(generator, "generate", "null " + cellName + " "
                    + nodeName + " " + serverName + " " + propagate)
        #endIf
    #endFor
#endDef

#-------------------------------------------------------------------------------
# addOrUpdateConnectorModule()
#
# Adds or updates a connector module.
#
# PARAMETERS
#   appName:            The application name.
#   srcPath:            The absolute path to the RAR file.
#   moduleURI:          The module URI.
#   moduleName:         The module name.
#-------------------------------------------------------------------------------
def addOrUpdateConnectorModule(appName, srcPath, moduleURI, moduleName=None):
    ddPath = "META-INF/ra.xml"
    updateParams = ["-operation", "addupdate", "-contents", srcPath,
            "-contenturi", moduleURI,]
    if (moduleName != None):
        targetServerMapping = getTargetServerMapping(appName, "true")
        if (targetServerMapping != ""):
            mapModulesToServersParams = [[moduleName, moduleURI + "," + ddPath,
                    targetServerMapping]]
            updateParams.append("-MapModulesToServers")
            updateParams.append(mapModulesToServersParams)
        #endIf
    #endIf
    AdminApp.update(appName, "modulefile", updateParams)
#endDef

#-------------------------------------------------------------------------------
# addOrUpdateEJBModule()
#
# Adds or updates an EJB module.
#
# PARAMETERS
#   appName:            The application name.
#   srcPath:            The absolute path to the EJB JAR file.
#   moduleURI:          The module URI.
#   jndiName:           The datasource and connection factory JNDI name.
#   moduleName:         The module name.
#   mappingOptionList:  The list of mapping options.
#   tokenDict:          The dictionary that contains token replacement values.
#-------------------------------------------------------------------------------
def addOrUpdateEJBModule(appName, srcPath, moduleURI, jndiName,
        moduleName=None, mappingOptionList=None, tokenDict=None):
    ddPath = "META-INF/ejb-jar.xml"
    updateParams = ["-operation", "addupdate", "-contents", srcPath,
            "-usedefaultbindings", "-contenturi", moduleURI,
            "-defaultbinding.datasource.jndi", jndiName,
            "-defaultbinding.cf.jndi", jndiName, "-defaultbinding.cf.resauth",
            "Per_Connection_Factory", "-custom",
            "com.ibm.ws.webservices.validateEJBJAR=false"]
    if (moduleName != None):
        targetServerMapping = getTargetServerMapping(appName, "true")
        if (targetServerMapping != ""):
            mapModulesToServersParams = [[moduleName, moduleURI + "," + ddPath,
                    targetServerMapping]]
            updateParams.append("-MapModulesToServers")
            updateParams.append(mapModulesToServersParams)
        #endIf
    #endIf
    if (mappingOptionList != None):
        mappingOptionDict = getMappingOptions(moduleName, moduleURI, ddPath,
                mappingOptionList, tokenDict)
        if (len(mappingOptionDict) != 0):
            for option in mappingOptionDict.keys():
                updateParams.append("-" + option)
                updateParams.append(mappingOptionDict[option])
            #endFor
        #endIf
    #endIf
    AdminApp.update(appName, "modulefile", updateParams)
#endDef

#-------------------------------------------------------------------------------
# addOrUpdateWebModule()
#
# Adds or updates a Web module.
#
# PARAMETERS
#   appName:            The application name.
#   srcPath:            The absolute path to the WAR file.
#   moduleURI:          The module URI.
#   contextRoot:        The context root. If not provided, default will be used.
#   virtualHost:        The virtual host. If not provided, default will be used.
#   moduleName:         The module name.
#-------------------------------------------------------------------------------
def addOrUpdateWebModule(appName, srcPath, moduleURI, contextRoot,
        virtualHost=None, moduleName=None):
    ddPath = "WEB-INF/web.xml"
    updateParams = ["-operation", "addupdate", "-contents", srcPath,
            "-contenturi", moduleURI]
    if (contextRoot != ""):
        updateParams.append("-contextroot")
        updateParams.append(contextRoot)
    #endIf
    if (virtualHost != None and virtualHost != ""):
        mapWebModToVHParams = [[moduleName, moduleURI + "," + ddPath,
                virtualHost]]
        updateParams.append("-MapWebModToVH")
        updateParams.append(mapWebModToVHParams)
    #endIf
    if (moduleName != None):
        targetServerMapping = getTargetServerMapping(appName, "true")
        if (targetServerMapping != ""):
            mapModulesToServersParams = [[moduleName, moduleURI + "," + ddPath,
                    targetServerMapping]]
            updateParams.append("-MapModulesToServers")
            updateParams.append(mapModulesToServersParams)
        #endIf
    #endIf
    AdminApp.update(appName, "modulefile", updateParams)
#endDef

#-------------------------------------------------------------------------------
# addWebModule()
#
# Adds or updates a Web module.
#
# Note: This function is retained for compatibility reasons. It does not set
#       virtual hosts and requires an extra call to the mapModuleToServers()
#       function to map the Web module to the target server mapping. It is
#       recommended that you use the addOrUpdateWebModule() above function
#       instead.
#
# PARAMETERS
#   appName:            The application name.
#   srcPath:            The absolute path to the WAR file.
#   moduleURI:          The module URI.
#   contextRoot:        The context root. If not provided, default will be used.
#-------------------------------------------------------------------------------
def addWebModule(appName, srcPath, moduleURI, contextRoot):
    addOrUpdateWebModule(appName, srcPath, moduleURI, contextRoot)
#endDef

#-------------------------------------------------------------------------------
# updateWithPartialApp()
#
# Updates the application with a partial application zip file.
#
# PARAMETERS
#   appName:            The application name.
#   srcPath:            The absolute path to the partial application zip file.
#-------------------------------------------------------------------------------
def updateWithPartialApp(appName, srcPath, partialAppFullUpdate="true"):
    updateParams = ["-contents", srcPath, "-custom",
            "com.ibm.websphere.management.application.fullupdate="
            + partialAppFullUpdate]
    AdminApp.update(appName, "partialapp", updateParams)
#endDef

#-------------------------------------------------------------------------------
# deleteModule()
#
# Deletes a module from the application.
#
# PARAMETERS
#   appName:            The application name.
#   moduleURI:          The module URI.
#-------------------------------------------------------------------------------
def deleteModule(appName, moduleURI):
    updateParams = ["-operation", "delete", "-contenturi", moduleURI]
    AdminApp.update(appName, "modulefile", updateParams)

#-------------------------------------------------------------------------------
# mapModuleToServer()
#
# Maps the given module to the target server mapping.
#
# PARAMETERS
#   appName:            The application name.
#   mapToWebServer:     "true" if web server mapping should be included; "false"
#                       otherwise.
#   moduleName:         The module name.
#   moduleURI:          The module URI.
#   ddPath:             The path to the deployment descriptor.
#-------------------------------------------------------------------------------
def mapModuleToServers(appName, mapToWebServer, moduleName, moduleURI, ddPath):
    targetServerMapping = getTargetServerMapping(appName, mapToWebServer)
    editParams = ["-MapModulesToServers", [[moduleName,
            moduleURI + "," + ddPath, targetServerMapping]]]
    if (targetServerMapping != ""):
        AdminApp.edit(appName, editParams)
    #endIf
#endDef

#-------------------------------------------------------------------------------
# createOrUpdateAndMapJ2CCF()
#
# Creates and maps a J2C connection factory for the embedded resource adapter
# of a connector module. Optionally any existing CF with the same JNDI name can
# be replaced by a new one.
#
# PARAMETERS
#   appName:            The application name.
#   moduleName:         The module name.
#   overwrite:          "true" if CF w/ same JNDI name should be recreated;
#                       "false" otherwise.
#   jndi:               The J2C CF JNDI name.
#   description:        The description/display name of the J2C CF.
#-------------------------------------------------------------------------------
def createOrUpdateAndMapJ2CCF(appName, moduleName, overwrite, jndiName,
        description):
    module = getModule(appName, moduleName)
    resourceAdapter = AdminConfig.showAttribute(module, "resourceAdapter")
    cfList = wsadminToList(
            AdminConfig.list("J2CConnectionFactory", resourceAdapter))
    for cf in cfList:
        cfJndiName = AdminConfig.showAttribute(cf, "jndiName")
        if (cfJndiName == jndiName):
            if (overwrite == "true"):
                AdminConfig.remove(cf)
            #endIf
            else:
                print("NOTE: A J2C connection factory with the JNDI name \""
                        + jndiName + "\" already exists in the resource "
                        + "adapter of the connector module \"" + moduleName
                        + "\" in the application \"" + appName
                        + "\". A new connection factory will NOT be created.")
                return
            #endElse
        elif (cfJndiName == "eis/javax.resource.cci.ConnectionFactory"):
            AdminConfig.remove(cf)
        #endElif
    #endFor
    newCf = AdminConfig.create("J2CConnectionFactory", resourceAdapter,
            [["name", moduleName], ["description", description],
            ["jndiName", jndiName]])
    mappingModuleAttrs = [["mappingConfigAlias", "DefaultPrincipalMapping"],
            ["authDataAlias", ""]]
    AdminConfig.create("MappingModule", newCf, mappingModuleAttrs)
#endDef

#-------------------------------------------------------------------------------
# mapWebModuleToVirtualHost()
#
# Maps a Web module to a virtual host.
#
# PARAMETERS
#   appName:            The application name.
#   moduleName:         The module name.
#   moduleURI:          The module URI.
#   vhName:             The virtual host name.
#-------------------------------------------------------------------------------
def mapWebModuleToVirtualHost(appName, moduleName, moduleURI, vhName):
    editParams = ["-MapWebModToVH", [[moduleName,
            moduleURI + ",WEB-INF/web.xml", vhName]]]
    AdminApp.edit(appName, editParams)
#endDef

#-------------------------------------------------------------------------------
# setStartingWeight()
#
# Sets the starting weight of a module in the application.
#
# PARAMETERS
#   appName:            The application name.
#   moduleURI:          The module URI.
#   startingWeight:     The starting weight.
#-------------------------------------------------------------------------------
def setStartingWeight(appName, moduleURI, startingWeight):
    module = getModule(appName, moduleURI)
    if (module != ""):
        AdminConfig.modify(module, [["startingWeight", startingWeight]])
    #endIf
#endDef

#-------------------------------------------------------------------------------
# save()
#
# Saves the configuration, and performs a cluster node synchronization
# (application rollout update) if appropriate arguments are provided.
#
# PARAMETERS
#   rolloutUpdate:      "true" if rollout update should be used; "false"
#                       otherwise.
#   appName:            The application name. This parameter will not be used if
#                       the "rolloutUpdate" parameter is set to "false".
#   timeout:            The timeout value in seconds for each node
#                       synchronization. This parameter will not be used if the
#                       "rolloutUpdate" parameter is set to "false".
#-------------------------------------------------------------------------------
def save(rolloutUpdate, appName, timeout):
    AdminConfig.save()
    if (rolloutUpdate == "true"):
        AdminTask.updateAppOnCluster("-ApplicationNames " + appName
                + " -timeout " + timeout)
    #endif
#endDef

#-------------------------------------------------------------------------------
# parseCSVFile()
#
# Parses a CSV file as a list of rows, each of which is a list of fields. Note
# that newline character in a field is not supported by this function.
#
# PARAMETERS
#   csvFile:            The CSV file to parse.
#
# RETURNS
#   A two-dimensional list representation of the CSV file.
#-------------------------------------------------------------------------------
def parseCSVFile(csvFile):
    rowList = []
    reader = open(csvFile)
    for row in reader.readlines():
        fieldList = []
        # Manually strip out newline characters since rstrip in the Jython
        # version WAS ships does not take argument and by itself the rstrip
        # method also removes whitespaces
        if row.endswith("\n"):
            row = row[0:-1]
        #endIf
        if row.endswith("\r"):
            row = row[0:-1]
        #endIf
        for field in re.split(",(?!(?:[^\",]|[^\"],[^\"])+\")", row):
            # Remove quotes from quoted fields
            if (field.startswith("\"") and field.endswith("\"")):
                field = field[1:-1]
            #endIf
            # Convert escaped quotes back to original
            field = field.replace("\"\"", "\"")
            fieldList.append(field)
        rowList.append(fieldList)
    #endFor
    reader.close()
    return rowList
#endDef

#-------------------------------------------------------------------------------
# validateMappingOptions()
#
#   Validate that the given mapping options are supported and the number of
#   parameters are correct.
#
# PARAMETERS
#   mappingOptionList:  The list of mapping options.
#
# RETURNS
#   1 if validation passes; 0 otherwise.
#-------------------------------------------------------------------------------
def validateMappingOptions(mappingOptionList):
    mappingOptionDict = {}
    for row in mappingOptionList:
        params = []
        option = row[1]
        numParams = len(row)
        if (numParams <= 1):
            wasOrWere = "was"
        #endIf
        else:
            wasOrWere = "were"
        #endElse
        if (option == "MapEJBRefToEJB"):
            if (numParams != 6):
                print("ERROR: The MapEJBRefToEJB option requires 6 parameters "
                        + "but " + str(numParams) + " " + wasOrWere
                        + " specified.")
                return 0
            #endIf
        #endIf
        elif (option == "MapResRefToEJB"):
            if (numParams != 9):
                print("ERROR: The MapResRefToEJB option requires 9 parameters "
                        + "but " + str(numParams) + " " + wasOrWere
                        + " specified.")
                return 0
            #endIf
        #endElif
        else:
            print("ERROR: The specified option " + option + " is not "
                    + "supported.")
            return 0
    #endFor
    return 1
#endDef

#-------------------------------------------------------------------------------
# getMappingOptions()
#
#   Returns a dictionary that contains the mapping option name and the required
#   parameters in a form consumable by the AdminApp.update() command for the
#   given module.
#
# PARAMETERS
#   moduleName:         The module name.
#   moduleURI:          The module URI.
#   ddPath:             The path to the deployment descriptor.
#   mappingOptionList:  The list of mapping options.
#   tokenDict:          The dictionary that contains token replacement values.
#
# RETURNS
#   A dictionary that contains the mapping option name and the required
#   parameters in a form consumable by the AdminApp.update() command for the
#   given module.
#-------------------------------------------------------------------------------
def getMappingOptions(moduleName, moduleURI, ddPath, mappingOptionList,
        tokenDict):
    mappingOptionDict = {}
    for row in mappingOptionList:
        if (row[0] == moduleName):
            params = []
            option = row[1]
            if (option == "MapEJBRefToEJB"):
                # Module name
                params.append(moduleName)
                # EJB name
                params.append(row[2])
                # Module deployment descriptor URI
                params.append(moduleURI + "," + ddPath)
                # Resource reference
                params.append(row[3])
                # Class
                params.append(row[4])
                # Target resource JNDI name
                params.append(row[5])
            #endIf
            elif (option == "MapResRefToEJB"):
                # Module name
                params.append(moduleName)
                # EJB name
                params.append(row[2])
                # Module deployment descriptor URI
                params.append(moduleURI + "," + ddPath)
                # Resource reference
                params.append(row[3])
                # Resource type
                params.append(row[4])
                # Target resource JNDI name
                params.append(row[5])
                # Login configuration name
                params.append(row[6])
                # Properties
                params.append(row[7])
                # Extended data source properties
                params.append(row[8])
            #endElif
            if (tokenDict != None):
                replaceInList(tokenDict, params)
            #endIf
            if (not option in mappingOptionDict.keys()):
                mappingOptionDict[option] = [params]
            #endIf
            else:
                mappingOptionDict[option].append(params)
            #endElse
        #endIf
    #endFor
    return mappingOptionDict
#endDef

#-------------------------------------------------------------------------------
# replaceInList()
#
# Replace values in a list given a dictionary that contains token replacement
# values.
#
# PARAMETERS
#   dictionary:         The dictionary that contains token replacement values.
#   list:               The list to replace values from
#-------------------------------------------------------------------------------
def replaceInList(dictionary, list):
    for i in range(len(list)):
        if (list[i] in dictionary.keys()):
            list[i] = dictionary[list[i]]
        #endIf
    #endFor
#endDef

#-------------------------------------------------------------------------------
# whatEnv()
#
# Check if base or nd environment.
#
# RETURNS
#   'nd' if connected to a dmgr, and
#   'base' otherwise
#-------------------------------------------------------------------------------
def whatEnv():
    servers = getObjectsOfType('Server')
    for server in servers:
        servertype = getObjectAttribute(server, 'serverType')
        if servertype == 'DEPLOYMENT_MANAGER':
            return 'nd'  # we have a deployment manager
        #endIf
    #endFor
    return 'base'  # no deployment manager, must be base
#endDef
    
#-------------------------------------------------------------------------------
# getObjectAttribute()
#
# PARAMETERS
#   objectid:      The ID of the WAS object
#   attributename: The name of the attribute that you wnt to extract from the object 
#
# RETURNS
#   The value of the named attribute of the config object with the given ID.
#   If there's no such attribute, returns None.
#   If the attribute value looks like a list, converts it to a real python list.
#
#-------------------------------------------------------------------------------
def getObjectAttribute(objectid, attributename):
    result = AdminConfig.showAttribute(objectid, attributename)
    if result != None and result.startswith("[") and result.endswith("]"):
        # List looks like "[value1 value2 value3]"
        result = _splitlist(result)
    #endif
    return result
#endDef

#-------------------------------------------------------------------------------
# getObjectsOfType()
#
# RETURNS
#   A python list of objectids of all WAS objects of the given type in the given WAS scope
#   (another object ID, e.g. a node's object id to limit the response to objects in that node)
#   Leaving scope default to None gets everything in the Cell with that type.
#   ALWAYS RETURNS A LIST EVEN IF ONLY ONE OBJECT.
#-------------------------------------------------------------------------------
def getObjectsOfType(typename, scope = None):
    if scope:
        return _splitlines(AdminConfig.list(typename, scope))
    else:
        return _splitlines(AdminConfig.list(typename))
    #endIf
#endDef

#-------------------------------------------------------------------------------
# Not user callable
#-------------------------------------------------------------------------------
def _splitlines(s):
  rv = [s]
  if '\r' in s:
    rv = s.split('\r\n')
  elif '\n' in s:
    rv = s.split('\n')
  #endIf
  if rv[-1] == '':
    rv = rv[:-1]
  #endIf
  return rv
#endDef

#-------------------------------------------------------------------------------
# Not user callable
#-------------------------------------------------------------------------------
  def _splitlist(s):
    if s[0] != '[' or s[-1] != ']':
        raise "Invalid string: %s" % s
    #endif
    return s[1:-1].split(' ')
#endDef
