package com.ibm.commerce.wc.update.utils;

/* 
 *------------------------------------------------------------------- 
 * IBM Confidential 
 * OCO Source Materials 
 * 
 * WebSphere Commerce 
 * 
 * (c) Copyright International Business Machines Corporation. 
 *     2006 
 *     All rights reserved. 
 * 
 * The source code for this program is not published or otherwise 
 * divested of its trade secrets, irrespective of what has been 
 * deposited with the US Copyright Office. 
 *------------------------------------------------------------------- 
 */

import java.io.File;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

/**
 * This is an Ant task wrapper to <code>XMLUpdater</code>.
 */
public class XMLUpdateTask extends Task {

	/**
	 * IBM copyright notice field.
	 */
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

	/**
	 * The XML config file.
	 */
	private File fConfigFile = null;

	/**
	 * The directory in which the input and output files are located, if they
	 * are given as relative paths.
	 */
	private File fDir = null;

	/**
	 * The path to the input file. When set, it will override the value defined
	 * in the configuration XML.
	 */
	private String fInputPath = null;

	/**
	 * The path to the output file. When set, it will override the value defined
	 * in the configuration XML.
	 */
	private String fOutputPath = null;

	/**
	 * The nested XML document inside this task for <code>XMLUpdater</code>.
	 */
	private String fCdata = null;

	/**
	 * Sets the XML config file.
	 * 
	 * @param configFile
	 *            The XML config file.
	 */
	public void setCfgFile(File configFile) {
		fConfigFile = configFile;
	}

	/**
	 * Sets the directory in which the input and output files are located.
	 * 
	 * @param dir
	 *            The directory in which the input and output files are located.
	 */
	public void setDir(File dir) {
		fDir = dir;
	}

	/**
	 * Sets the path to the input file. When set, it will override the value
	 * defined in the configuration XML.
	 * 
	 * @param inputPath
	 *            The path to the input file.
	 */
	public void setInputPath(String inputPath) {
		fInputPath = inputPath;
	}

	/**
	 * Sets the path to the output file. When set, it will override the value
	 * defined in the configuration XML.
	 * 
	 * @param outputPath
	 *            The path to the output file.
	 */
	public void setOutputPath(String outputPath) {
		fOutputPath = outputPath;
	}

	/**
	 * Sets nested CDATA elements.
	 * 
	 * @param cdata
	 *            CDATA nested in the task element.
	 */
	public void addText(String cdata) {
		if (fCdata == null) {
			fCdata = "";
		}
		fCdata = fCdata + getProject().replaceProperties(cdata);
	}

	/**
	 * Validates input parameters.
	 * 
	 * @throws BuildException
	 */
	public void validate() throws BuildException {
		if (fConfigFile == null && fCdata == null) {
			throw new BuildException("Either the attribute \"configFile\" or "
					+ "CDATA must be specified.");
		}
		if ((fConfigFile != null && fCdata != null)) {
			throw new BuildException("Only one of attribute \"configFile\" or "
					+ "CDATA can be specified.");
		}
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see org.apache.tools.ant.Task#execute()
	 */
	public void execute() throws BuildException {
		validate();
		try {
			XMLUpdater updater = null;
			if (fCdata != null && fCdata.length() != 0) {
				Document doc = DocumentBuilderFactory.newInstance()
						.newDocumentBuilder().parse(
								new InputSource(new StringReader(fCdata)));
				updater = (fDir != null) ? new XMLUpdater(doc, fDir)
						: new XMLUpdater(doc);
			} else {
				updater = (fDir != null) ? new XMLUpdater(fConfigFile, fDir)
						: new XMLUpdater(fConfigFile);
			}
			updater.setLimitToSingleFile(true);
			if (fInputPath != null) {
				log("Input path defined in the XML configuration will be overridden by "
						+ fInputPath);
				updater.setInputPath(fInputPath);
			}
			if (fOutputPath != null) {
				log("Output path defined in the XML configuration will be overridden by "
						+ fOutputPath);
				updater.setOutputPath(fOutputPath);
			}
			updater.setTask(this);
			updater.updateAll();
		} catch (Exception e) {
			throw new BuildException(e);
		}
	}
}
